@@ -1,5 +1,6 @@
 /* poppler-attachment.cc: glib wrapper for poppler
  * Copyright (C) 2006, Red Hat, Inc.
+ * Copyright (C) 2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -37,7 +38,7 @@
 
 struct PopplerAttachmentPrivate
 {
-    Object obj_stream {};
+    Object obj_stream;
     GDateTime *mtime;
     GDateTime *ctime;
 };
@@ -69,16 +70,19 @@ static void poppler_attachment_finalize(GObject *obj)
     attachment = (PopplerAttachment *)obj;
     priv = GET_PRIVATE(attachment);
 
-    if (attachment->name)
+    if (attachment->name) {
         g_free(attachment->name);
+    }
     attachment->name = nullptr;
 
-    if (attachment->description)
+    if (attachment->description) {
         g_free(attachment->description);
+    }
     attachment->description = nullptr;
 
-    if (attachment->checksum)
+    if (attachment->checksum) {
         g_string_free(attachment->checksum, TRUE);
+    }
     attachment->checksum = nullptr;
 
     g_clear_pointer(&priv->mtime, g_date_time_unref);
@@ -102,10 +106,12 @@ PopplerAttachment *_poppler_attachment_new(FileSpec *emb_file)
     attachment = (PopplerAttachment *)g_object_new(POPPLER_TYPE_ATTACHMENT, nullptr);
     priv = GET_PRIVATE(attachment);
 
-    if (emb_file->getFileName())
+    if (emb_file->getFileName()) {
         attachment->name = _poppler_goo_string_to_utf8(emb_file->getFileName());
-    if (emb_file->getDescription())
+    }
+    if (emb_file->getDescription()) {
         attachment->description = _poppler_goo_string_to_utf8(emb_file->getDescription());
+    }
 
     embFile = emb_file->getEmbeddedFile();
     if (embFile != nullptr && embFile->streamObject()->isStream()) {
@@ -128,8 +134,9 @@ PopplerAttachment *_poppler_attachment_new(FileSpec *emb_file)
             G_GNUC_END_IGNORE_DEPRECATIONS
         }
 
-        if (embFile->checksum() && embFile->checksum()->getLength() > 0)
-            attachment->checksum = g_string_new_len(embFile->checksum()->c_str(), embFile->checksum()->getLength());
+        if (embFile->checksum() && !embFile->checksum()->empty()) {
+            attachment->checksum = g_string_new_len(embFile->checksum()->c_str(), embFile->checksum()->size());
+        }
         priv->obj_stream = embFile->streamObject()->copy();
     } else {
         g_warning("Missing stream object for embedded file");
@@ -352,7 +359,9 @@ gboolean poppler_attachment_save_to_callback(PopplerAttachment *attachment, Popp
     priv = GET_PRIVATE(attachment);
 
     stream = priv->obj_stream.getStream();
-    stream->reset();
+    if (!stream->reset()) {
+        return FALSE;
+    }
 
     do {
         int data;
@@ -367,8 +376,9 @@ gboolean poppler_attachment_save_to_callback(PopplerAttachment *attachment, Popp
         }
 
         if (i > 0) {
-            if (!(save_func)(buf, i, user_data, error))
+            if (!(save_func)(buf, i, user_data, error)) {
                 return FALSE;
+            }
         }
     } while (!eof_reached);
 
