@@ -39,6 +39,7 @@
 // Copyright (C) 2024 Vincent Lefevre <vincent@vinc17.net>
 // Copyright (C) 2024 Dmitry Shubin <dshubin@accusoft.com>
 // Copyright (C) 2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+// Copyright (C) 2025 Nelson Benítez León <nbenitezl@gmail.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -214,14 +215,11 @@ CairoFreeTypeFont *CairoFreeTypeFont::create(const std::shared_ptr<GfxFont> &gfx
     std::optional<GfxFontLoc> fontLoc;
     char **enc;
     const char *name;
-    FoFiType1C *ff1c;
     std::optional<FreeTypeFontFace> font_face;
     std::vector<int> codeToGID;
     bool substitute = false;
 
     Ref ref = *gfxFont->getID();
-    Ref embFontID = Ref::INVALID();
-    gfxFont->getEmbeddedFontID(&embFontID);
     GfxFontType fontType = gfxFont->getType();
 
     if (!(fontLoc = gfxFont->locateFont(xref, nullptr))) {
@@ -285,7 +283,7 @@ CairoFreeTypeFont *CairoFreeTypeFont::create(const std::shared_ptr<GfxFont> &gfx
         } else {
             std::unique_ptr<FoFiTrueType> ff;
             if (!font_data.empty()) {
-                ff = FoFiTrueType::make(font_data.data(), font_data.size(), faceIndex);
+                ff = FoFiTrueType::make(std::span(font_data), faceIndex);
             } else {
                 ff = FoFiTrueType::load(fileName.c_str(), faceIndex);
             }
@@ -300,7 +298,7 @@ CairoFreeTypeFont *CairoFreeTypeFont::create(const std::shared_ptr<GfxFont> &gfx
     case fontTrueTypeOT: {
         std::unique_ptr<FoFiTrueType> ff;
         if (!font_data.empty()) {
-            ff = FoFiTrueType::make(font_data.data(), font_data.size(), faceIndex);
+            ff = FoFiTrueType::make(std::span(font_data), faceIndex);
         } else {
             ff = FoFiTrueType::load(fileName.c_str(), faceIndex);
         }
@@ -323,15 +321,15 @@ CairoFreeTypeFont *CairoFreeTypeFont::create(const std::shared_ptr<GfxFont> &gfx
     case fontCIDType0:
     case fontCIDType0C:
         if (!useCIDs) {
+            std::unique_ptr<FoFiType1C> ff1c;
             if (!font_data.empty()) {
-                ff1c = FoFiType1C::make(font_data.data(), font_data.size());
+                ff1c = FoFiType1C::make(std::span(font_data));
             } else {
                 ff1c = FoFiType1C::load(fileName.c_str());
             }
             if (ff1c) {
                 std::vector<int> src = ff1c->getCIDToGIDMap();
                 codeToGID = std::move(src);
-                delete ff1c;
             }
         }
 
@@ -352,7 +350,7 @@ CairoFreeTypeFont *CairoFreeTypeFont::create(const std::shared_ptr<GfxFont> &gfx
             if (!useCIDs) {
                 std::unique_ptr<FoFiTrueType> ff;
                 if (!font_data.empty()) {
-                    ff = FoFiTrueType::make(font_data.data(), font_data.size(), faceIndex);
+                    ff = FoFiTrueType::make(std::span(font_data), faceIndex);
                 } else {
                     ff = FoFiTrueType::load(fileName.c_str(), faceIndex);
                 }
@@ -380,6 +378,9 @@ CairoFreeTypeFont *CairoFreeTypeFont::create(const std::shared_ptr<GfxFont> &gfx
     return new CairoFreeTypeFont(ref, font_face->cairo_font_face, std::move(codeToGID), substitute);
 
 err2:
+    if (fontLoc && fontLoc->locType == gfxFontLocEmbedded) {
+        gfxFont->invalidateEmbeddedFont();
+    }
     fprintf(stderr, "some font thing failed\n");
     return nullptr;
 }
@@ -414,7 +415,7 @@ static cairo_status_t _init_type3_glyph(cairo_scaled_font_t *scaled_font, cairo_
     type3_font_info_t *info;
 
     info = (type3_font_info_t *)cairo_font_face_get_user_data(cairo_scaled_font_get_font_face(scaled_font), &type3_font_key);
-    const double *mat = info->font->getFontBBox();
+    const std::array<double, 4> &mat = info->font->getFontBBox();
     extents->ascent = mat[3]; /* y2 */
     extents->descent = -mat[3]; /* -y1 */
     extents->height = extents->ascent + extents->descent;
@@ -429,7 +430,6 @@ static cairo_status_t _render_type3_glyph(cairo_scaled_font_t *scaled_font, unsi
     Dict *charProcs;
     Object charProc;
     cairo_matrix_t matrix, invert_y_axis;
-    const double *mat;
     double wx, wy;
     type3_font_info_t *info;
     Gfx *gfx;
@@ -448,7 +448,7 @@ static cairo_status_t _render_type3_glyph(cairo_scaled_font_t *scaled_font, unsi
         return CAIRO_STATUS_SUCCESS;
     }
 
-    mat = info->font->getFontMatrix();
+    const std::array<double, 6> &mat = info->font->getFontMatrix();
     matrix.xx = mat[0];
     matrix.yx = mat[1];
     matrix.xy = mat[2];
@@ -530,7 +530,6 @@ CairoType3Font *CairoType3Font::create(const std::shared_ptr<GfxFont> &gfxFont,
 {
     std::vector<int> codeToGID;
     char *name;
-    const double *mat;
 
     Dict *charProcs = std::static_pointer_cast<Gfx8BitFont>(gfxFont)->getCharProcs();
     Ref ref = *gfxFont->getID();
@@ -548,7 +547,7 @@ CairoType3Font *CairoType3Font::create(const std::shared_ptr<GfxFont> &gfxFont,
     output_dev->setPrinting(printing);
 
     Dict *resDict = std::static_pointer_cast<Gfx8BitFont>(gfxFont)->getResources();
-    mat = gfxFont->getFontBBox();
+    const std::array<double, 4> &mat = gfxFont->getFontBBox();
     PDFRectangle box;
     box.x1 = mat[0];
     box.y1 = mat[1];
