@@ -689,7 +689,7 @@ StitchingFunction::StitchingFunction(Object *funcObj, Dict *dict, RefRecursionCh
         error(errSyntaxError, -1, "Missing 'Functions' entry in stitching function");
         return;
     }
-    k = obj1.arrayGetLength();
+    const int k = obj1.arrayGetLength();
     funcs.resize(k);
     bounds = (double *)gmallocn(k + 1, sizeof(double));
     encode = (double *)gmallocn(2 * k, sizeof(double));
@@ -759,13 +759,12 @@ StitchingFunction::StitchingFunction(Object *funcObj, Dict *dict, RefRecursionCh
 
 StitchingFunction::StitchingFunction(const StitchingFunction *func, PrivateTag) : Function(func)
 {
-    k = func->k;
-
-    funcs.resize(func->funcs.size());
-    for (int i = 0; i < k; ++i) {
-        funcs[i] = func->funcs[i]->copy();
+    funcs.reserve(func->funcs.size());
+    for (const std::unique_ptr<Function> &f : func->funcs) {
+        funcs.push_back(f->copy());
     }
 
+    const int k = funcs.size();
     bounds = (double *)gmallocn(k + 1, sizeof(double));
     memcpy(bounds, func->bounds, (k + 1) * sizeof(double));
 
@@ -797,6 +796,7 @@ void StitchingFunction::transform(const double *in, double *out) const
     } else {
         x = in[0];
     }
+    const int k = funcs.size();
     for (i = 0; i < k - 1; ++i) {
         if (x < bounds[i + 1]) {
             break;
@@ -1092,6 +1092,7 @@ PostScriptFunction::PostScriptFunction(Object *funcObj, Dict *dict)
     code = nullptr;
     codeSize = 0;
     ok = false;
+    int recursionCounter = 0;
 
     //----- initialize the generic stuff
     if (!init(dict)) {
@@ -1120,7 +1121,7 @@ PostScriptFunction::PostScriptFunction(Object *funcObj, Dict *dict)
         goto err1;
     }
     codePtr = 0;
-    if (!parseCode(str, &codePtr)) {
+    if (!parseCode(str, &codePtr, recursionCounter)) {
         goto err2;
     }
     str->close();
@@ -1207,12 +1208,17 @@ void PostScriptFunction::transform(const double *in, double *out) const
     }
 }
 
-bool PostScriptFunction::parseCode(Stream *str, int *codePtr)
+bool PostScriptFunction::parseCode(Stream *str, int *codePtr, int &recursionCounter)
 {
     bool isReal;
     int opPtr, elsePtr;
     int a, b, mid, cmp;
 
+    if (recursionCounter > 1024) {
+        error(errSyntaxError, -1, "PostScript function recursion too deep");
+        return false;
+    }
+
     while (true) {
         // This needs to be on the heap to help make parseCode
         // able to call itself more times recursively
@@ -1239,13 +1245,13 @@ bool PostScriptFunction::parseCode(Stream *str, int *codePtr)
             opPtr = *codePtr;
             *codePtr += 3;
             resizeCode(opPtr + 2);
-            if (!parseCode(str, codePtr)) {
+            if (!parseCode(str, codePtr, ++recursionCounter)) {
                 return false;
             }
             tok = getToken(str);
             if (!tok->cmp("{")) {
                 elsePtr = *codePtr;
-                if (!parseCode(str, codePtr)) {
+                if (!parseCode(str, codePtr, ++recursionCounter)) {
                     return false;
                 }
                 tok = getToken(str);
