@@ -21,6 +21,7 @@
 // Copyright (C) 2023, 2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
 // Copyright (C) 2023 Even Rouault <even.rouault@mines-paris.org>
 // Copyright (C) 2023 Oliver Sander <oliver.sander@tu-dresden.de>
+// Copyright (C) 2025 Jonathan Hähne <jonathan.haehne@hotmail.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -69,12 +70,12 @@ static const long long LongLongSafeLimit = (LLONG_MAX - 9) / 10;
 // Lexer
 //------------------------------------------------------------------------
 
-Lexer::Lexer(XRef *xrefA, Stream *str)
+Lexer::Lexer(XRef *xrefA, std::unique_ptr<Stream> &&str)
 {
     lookCharLastValueCached = LOOK_VALUE_NOT_CACHED;
     xref = xrefA;
 
-    curStr = Object(str);
+    curStr = Object(std::move(str));
     streams = new Array(xref);
     streams->add(curStr.copy());
     strPtr = 0;
@@ -162,6 +163,7 @@ int Lexer::lookChar()
 
 Object Lexer::getObj(int objNum)
 {
+    char tokBuf[tokBufSize]; // temporary token buffer
     char *p;
     int c, c2;
     bool comment, neg, done, overflownInteger, overflownLongLong;
@@ -175,7 +177,7 @@ Object Lexer::getObj(int objNum)
     comment = false;
     while (true) {
         if ((c = getChar()) == EOF) {
-            return Object(objEOF);
+            return Object::eof();
         }
         if (comment) {
             if (c == '\r' || c == '\n') {
@@ -302,13 +304,9 @@ Object Lexer::getObj(int objNum)
         do {
             c2 = EOF;
             switch (c = getChar()) {
-
+            // Should technically error on newlines too, but this breaks
+            // some PDF files, e.g., ones from Photoshop.
             case EOF:
-#if 0
-      // This breaks some PDF files, e.g., ones from Photoshop.
-      case '\r':
-      case '\n':
-#endif
                 error(errSyntaxError, getPos(), "Unterminated string");
                 done = true;
                 break;
@@ -412,6 +410,7 @@ Object Lexer::getObj(int objNum)
                 ++n;
             }
         } while (!done);
+
         if (n >= 0) {
             s.append(tokBuf, n);
             // Check utf8
@@ -420,7 +419,7 @@ Object Lexer::getObj(int objNum)
             }
             return Object(std::move(s));
         } else {
-            return Object(objEOF);
+            return Object::eof();
         }
         break;
     }
@@ -470,16 +469,15 @@ Object Lexer::getObj(int objNum)
                 // Somewhat arbitrary threshold
                 if (unlikely(n == 1024 * 1024)) {
                     error(errSyntaxError, getPos(), "Error: name token is larger than 1 MB. Suspicion of hostile file. Stopping parsing");
-                    return Object(objEOF);
+                    return Object::eof();
                 }
                 s.push_back((char)c);
             }
         }
         if (n < tokBufSize) {
-            *p = '\0';
-            return Object(objName, tokBuf);
+            return Object(objName, std::string_view(tokBuf, n));
         } else {
-            Object obj(objName, s.c_str());
+            Object obj(objName, s);
             return obj;
         }
         break;
@@ -487,11 +485,9 @@ Object Lexer::getObj(int objNum)
 
     // array punctuation
     case '[':
+        return Object(objCmd, std::string_view("[", 1));
     case ']':
-        tokBuf[0] = c;
-        tokBuf[1] = '\0';
-        return Object(objCmd, tokBuf);
-        break;
+        return Object(objCmd, std::string_view("]", 1));
 
     // hex string or dict punctuation
     case '<':
@@ -500,9 +496,7 @@ Object Lexer::getObj(int objNum)
         // dict punctuation
         if (c == '<') {
             getChar();
-            tokBuf[0] = tokBuf[1] = '<';
-            tokBuf[2] = '\0';
-            return Object(objCmd, tokBuf);
+            return Object(objCmd, std::string_view("<<", 2));
 
             // hex string
         } else {
@@ -557,12 +551,10 @@ Object Lexer::getObj(int objNum)
         c = lookChar();
         if (c == '>') {
             getChar();
-            tokBuf[0] = tokBuf[1] = '>';
-            tokBuf[2] = '\0';
-            return Object(objCmd, tokBuf);
+            return Object(objCmd, std::string_view(">>", 2));
         } else {
             error(errSyntaxError, getPos(), "Illegal character '>'");
-            return Object(objError);
+            return Object::error();
         }
         break;
 
@@ -571,7 +563,7 @@ Object Lexer::getObj(int objNum)
     case '{':
     case '}':
         error(errSyntaxError, getPos(), "Illegal character '{0:c}'", c);
-        return Object(objError);
+        return Object::error();
         break;
 
     // command
@@ -587,15 +579,15 @@ Object Lexer::getObj(int objNum)
             }
             *p++ = c;
         }
-        *p = '\0';
-        if (tokBuf[0] == 't' && !strcmp(tokBuf, "true")) {
+        std::string_view res(tokBuf, n);
+        if (res == "true") {
             return Object(true);
-        } else if (tokBuf[0] == 'f' && !strcmp(tokBuf, "false")) {
+        } else if (res == "false") {
             return Object(false);
-        } else if (tokBuf[0] == 'n' && !strcmp(tokBuf, "null")) {
-            return Object(objNull);
+        } else if (res == "null") {
+            return Object::null();
         } else {
-            return Object(objCmd, tokBuf);
+            return Object(objCmd, res);
         }
         break;
     }
@@ -603,9 +595,10 @@ Object Lexer::getObj(int objNum)
     return Object();
 }
 
-Object Lexer::getObj(const char *cmdA, int objNum)
+Object Lexer::getObj(std::string_view cmdA, int objNum)
 {
-    char *p;
+    char tokBuf[tokBufSize]; // temporary token buffer
+    char *p = tokBuf;
     int c;
     bool comment;
     int n;
@@ -614,10 +607,10 @@ Object Lexer::getObj(const char *cmdA, int objNum)
     comment = false;
     const char *cmd1 = tokBuf;
     *tokBuf = 0;
-    while ((strcmp(cmdA, cmd1) != 0) && (objNum < 0 || (xref && xref->getNumEntry(getPos()) == objNum))) {
+    while (cmdA != std::string_view(cmd1, p) && (objNum < 0 || (xref && xref->getNumEntry(getPos()) == objNum))) {
         while (true) {
             if ((c = getChar()) == EOF) {
-                return Object(objEOF);
+                return Object::eof();
             }
             if (comment) {
                 if (c == '\r' || c == '\n') {
@@ -639,10 +632,9 @@ Object Lexer::getObj(const char *cmdA, int objNum)
             }
             *p++ = c;
         }
-        *p = '\0';
     }
 
-    return Object(objCmd, tokBuf);
+    return Object(objCmd, std::string_view(tokBuf, p));
 }
 
 void Lexer::skipToNextLine()
