@@ -10,6 +10,7 @@
 // Copyright 2019 Adam Reichold <adam.reichold@t-online.de>
 // Copyright 2019 Even Rouault <even.rouault@spatialys.com>
 // Copyright 2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+// Copyright (C) 2025 Jonathan Hähne <jonathan.haehne@hotmail.com>
 //
 //========================================================================
 
@@ -26,7 +27,7 @@ Linearization::Linearization(BaseStream *str)
     Parser *parser;
 
     (void)str->reset();
-    parser = new Parser(nullptr, str->makeSubStream(str->getStart(), false, 0, Object(objNull)), false);
+    parser = new Parser(nullptr, str->makeSubStream(str->getStart(), false, 0, Object::null()), false);
     Object obj1 = parser->getObj();
     Object obj2 = parser->getObj();
     Object obj3 = parser->getObj();
@@ -51,7 +52,7 @@ unsigned int Linearization::getLength() const
     }
 
     int length;
-    if (linDict.getDict()->lookupInt("L", nullptr, &length) && length > 0) {
+    if (linDict.getDict()->lookupInt("L", {}, &length) && length > 0) {
         return length;
     } else {
         error(errSyntaxWarning, -1, "Length in linearization table is invalid");
@@ -128,7 +129,7 @@ unsigned int Linearization::getHintsLength2() const
 int Linearization::getObjectNumberFirst() const
 {
     int objectNumberFirst = 0;
-    if (linDict.isDict() && linDict.getDict()->lookupInt("O", nullptr, &objectNumberFirst) && objectNumberFirst > 0) {
+    if (linDict.isDict() && linDict.getDict()->lookupInt("O", {}, &objectNumberFirst) && objectNumberFirst > 0) {
         return objectNumberFirst;
     } else {
         error(errSyntaxWarning, -1, "Object number of first page in linearization table is invalid");
@@ -139,7 +140,7 @@ int Linearization::getObjectNumberFirst() const
 unsigned int Linearization::getEndFirst() const
 {
     int pageEndFirst = 0;
-    if (linDict.isDict() && linDict.getDict()->lookupInt("E", nullptr, &pageEndFirst) && pageEndFirst > 0) {
+    if (linDict.isDict() && linDict.getDict()->lookupInt("E", {}, &pageEndFirst) && pageEndFirst > 0) {
         return pageEndFirst;
     } else {
         error(errSyntaxWarning, -1, "First page end offset in linearization table is invalid");
@@ -150,7 +151,7 @@ unsigned int Linearization::getEndFirst() const
 int Linearization::getNumPages() const
 {
     int numPages = 0;
-    if (linDict.isDict() && linDict.getDict()->lookupInt("N", nullptr, &numPages) && numPages > 0) {
+    if (linDict.isDict() && linDict.getDict()->lookupInt("N", {}, &numPages) && numPages > 0) {
         return numPages;
     } else {
         error(errSyntaxWarning, -1, "Page count in linearization table is invalid");
@@ -161,7 +162,7 @@ int Linearization::getNumPages() const
 unsigned int Linearization::getMainXRefEntriesOffset() const
 {
     int mainXRefEntriesOffset = 0;
-    if (linDict.isDict() && linDict.getDict()->lookupInt("T", nullptr, &mainXRefEntriesOffset) && mainXRefEntriesOffset > 0) {
+    if (linDict.isDict() && linDict.getDict()->lookupInt("T", {}, &mainXRefEntriesOffset) && mainXRefEntriesOffset > 0) {
         return mainXRefEntriesOffset;
     } else {
         error(errSyntaxWarning, -1, "Main Xref offset in linearization table is invalid");
@@ -174,7 +175,7 @@ int Linearization::getPageFirst() const
     int pageFirst = 0; // Optional, defaults to 0.
 
     if (linDict.isDict()) {
-        linDict.getDict()->lookupInt("P", nullptr, &pageFirst);
+        linDict.getDict()->lookupInt("P", {}, &pageFirst);
     }
 
     if ((pageFirst < 0) || (pageFirst >= getNumPages())) {
