@@ -9,6 +9,7 @@
 // Copyright 2021, 2023 Adrian Johnson <ajohnson@redneon.com>
 // Copyright 2022 Oliver Sander <oliver.sander@tu-dresden.de>
 // Copyright 2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+// Copyright (C) 2025 Jonathan Hähne <jonathan.haehne@hotmail.com>
 //
 //========================================================================
 
@@ -31,7 +32,7 @@ MarkedContentOutputDev::~MarkedContentOutputDev() = default;
 
 void MarkedContentOutputDev::endSpan()
 {
-    if (currentText && currentText->getLength()) {
+    if (currentText && !currentText->empty()) {
         // The TextSpan takes ownership of currentText and
         // increases the reference count for currentFont.
         textSpans.push_back(TextSpan(std::move(currentText), currentFont, currentColor));
@@ -79,7 +80,7 @@ void MarkedContentOutputDev::beginMarkedContent(const char *name, Dict *properti
 {
     int id = -1;
     if (properties) {
-        properties->lookupInt("MCID", nullptr, &id);
+        properties->lookupInt("MCID", {}, &id);
     }
 
     if (id == -1) {
