@@ -14,10 +14,12 @@
 // under GPL version 2 or later
 //
 // Copyright (C) 2009 Carlos Garcia Campos <carlosgc@gnome.org>
-// Copyright (C) 2010, 2011, 2018-2020 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2010, 2011, 2018-2021, 2025 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2011, 2014 William Bader <williambader@hotmail.com>
 // Copyright (C) 2011, 2013 Thomas Freitag <Thomas.Freitag@alfa.de>
 // Copyright (C) 2011 Adrian Johnson <ajohnson@redneon.com>
+// Copyright (C) 2022 Oliver Sander <oliver.sander@tu-dresden.de>
+// Copyright (C) 2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -44,7 +46,7 @@ PreScanOutputDev::PreScanOutputDev(PSLevel levelA) : level(levelA)
     clearStats();
 }
 
-PreScanOutputDev::~PreScanOutputDev() { }
+PreScanOutputDev::~PreScanOutputDev() = default;
 
 void PreScanOutputDev::startPage(int /*pageNum*/, GfxState * /*state*/, XRef * /*xref*/) { }
 
@@ -52,13 +54,11 @@ void PreScanOutputDev::endPage() { }
 
 void PreScanOutputDev::stroke(GfxState *state)
 {
-    double *dash;
-    int dashLen;
     double dashStart;
 
     check(state->getStrokeColorSpace(), state->getStrokeColor(), state->getStrokeOpacity(), state->getBlendMode());
-    state->getLineDash(&dash, &dashLen, &dashStart);
-    if (dashLen != 0) {
+    const std::vector<double> &dash = state->getLineDash(&dashStart);
+    if (!dash.empty()) {
         gdi = false;
     }
 }
@@ -73,15 +73,14 @@ void PreScanOutputDev::eoFill(GfxState *state)
     check(state->getFillColorSpace(), state->getFillColor(), state->getFillOpacity(), state->getBlendMode());
 }
 
-bool PreScanOutputDev::tilingPatternFill(GfxState *state, Gfx *gfx, Catalog *catalog, Object *str, const double *pmat, int paintType, int /*tilingType*/, Dict *resDict, const double *mat, const double *bbox, int x0, int y0, int x1, int y1,
-                                         double xStep, double yStep)
+bool PreScanOutputDev::tilingPatternFill(GfxState *state, Gfx *gfx, Catalog *catalog, GfxTilingPattern *tPat, const std::array<double, 6> &mat, int x0, int y0, int x1, int y1, double xStep, double yStep)
 {
-    if (paintType == 1) {
+    if (tPat->getPaintType() == 1) {
         bool tilingNeeded = (x1 - x0 != 1 || y1 - y0 != 1);
         if (tilingNeeded) {
             inTilingPatternFill++;
         }
-        gfx->drawForm(str, resDict, mat, bbox);
+        gfx->drawForm(tPat->getContentStream(), tPat->getResDict(), mat, tPat->getBBox());
         if (tilingNeeded) {
             inTilingPatternFill--;
         }
@@ -152,7 +151,7 @@ void PreScanOutputDev::beginStringOp(GfxState *state)
         check(state->getStrokeColorSpace(), state->getStrokeColor(), state->getStrokeOpacity(), state->getBlendMode());
     }
 
-    const GfxFont *font = state->getFont();
+    std::shared_ptr<const GfxFont> font = state->getFont();
     state->getFontTransMat(&m11, &m12, &m21, &m22);
     //~ this should check for external fonts that are non-TrueType
     simpleTTF = fabs(m11 + m22) < 0.01 && m11 > 0 && fabs(m12) < 0.01 && fabs(m21) < 0.01 && fabs(state->getHorizScaling() - 1) < 0.001 && (font->getType() == fontTrueType || font->getType() == fontTrueTypeOT);
@@ -185,10 +184,13 @@ void PreScanOutputDev::drawImageMask(GfxState *state, Object * /*ref*/, Stream *
     }
 
     if (inlineImg) {
-        str->reset();
+        if (!str->reset()) {
+            return;
+        }
         j = height * ((width + 7) / 8);
-        for (i = 0; i < j; ++i)
+        for (i = 0; i < j; ++i) {
             str->getChar();
+        }
         str->close();
     }
 }
@@ -219,10 +221,13 @@ void PreScanOutputDev::drawImage(GfxState *state, Object * /*ref*/, Stream *str,
     }
 
     if (inlineImg) {
-        str->reset();
+        if (!str->reset()) {
+            return;
+        }
         j = height * ((width * colorMap->getNumPixelComps() * colorMap->getBits() + 7) / 8);
-        for (i = 0; i < j; ++i)
+        for (i = 0; i < j; ++i) {
             str->getChar();
+        }
         str->close();
     }
 }
@@ -267,17 +272,17 @@ void PreScanOutputDev::drawSoftMaskedImage(GfxState * /*state*/, Object * /*ref*
     gdi = false;
 }
 
-void PreScanOutputDev::beginTransparencyGroup(GfxState * /*state*/, const double * /*bbox*/, GfxColorSpace * /*blendingColorSpace*/, bool /*isolated*/, bool /*knockout*/, bool /*forSoftMask*/)
+void PreScanOutputDev::beginTransparencyGroup(GfxState * /*state*/, const std::array<double, 4> & /*bbox*/, GfxColorSpace * /*blendingColorSpace*/, bool /*isolated*/, bool /*knockout*/, bool /*forSoftMask*/)
 {
     gdi = false;
 }
 
-void PreScanOutputDev::paintTransparencyGroup(GfxState *state, const double * /*bbox*/)
+void PreScanOutputDev::paintTransparencyGroup(GfxState *state, const std::array<double, 4> & /*bbox*/)
 {
     check(state->getFillColorSpace(), state->getFillColor(), state->getFillOpacity(), state->getBlendMode());
 }
 
-void PreScanOutputDev::setSoftMask(GfxState * /*state*/, const double * /*bbox*/, bool /*alpha*/, Function * /*transferFunc*/, GfxColor * /*backdropColor*/)
+void PreScanOutputDev::setSoftMask(GfxState * /*state*/, const std::array<double, 4> & /*bbox*/, bool /*alpha*/, Function * /*transferFunc*/, GfxColor * /*backdropColor*/)
 {
     transparency = true;
 }
