@@ -15,7 +15,7 @@
 //
 // Copyright (C) 2005 Jeff Muizelaar <jeff@infidigm.net>
 // Copyright (C) 2008 Julien Rebetez <julien@fhtagn.net>
-// Copyright (C) 2008, 2010, 2011, 2016-2022, 2024 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2008, 2010, 2011, 2016-2022, 2024, 2025 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2009 Carlos Garcia Campos <carlosgc@gnome.org>
 // Copyright (C) 2009 Stefan Thomas <thomas@eload24.com>
 // Copyright (C) 2010 Hib Eris <hib@hiberis.nl>
@@ -311,7 +311,7 @@ public:
     // Put a char in the stream
     virtual void put(char c) = 0;
 
-    virtual size_t write(std::span<unsigned char> data) = 0;
+    virtual size_t write(std::span<const unsigned char> data) = 0;
 
     virtual void printf(const char *format, ...) GCC_PRINTF_FORMAT(2, 3) = 0;
 };
@@ -332,7 +332,7 @@ public:
 
     void put(char c) override;
 
-    size_t write(std::span<unsigned char> data) override;
+    size_t write(std::span<const unsigned char> data) override;
 
     void printf(const char *format, ...) override GCC_PRINTF_FORMAT(2, 3);
 
@@ -353,7 +353,7 @@ public:
     BaseStream(Object &&dictA, Goffset lengthA);
     ~BaseStream() override;
     virtual BaseStream *copy() = 0;
-    virtual Stream *makeSubStream(Goffset start, bool limited, Goffset length, Object &&dict) = 0;
+    virtual std::unique_ptr<Stream> makeSubStream(Goffset start, bool limited, Goffset length, Object &&dict) = 0;
     void setPos(Goffset pos, int dir = 0) override = 0;
     bool isBinary(bool last = true) const override { return last; }
     BaseStream *getBaseStream() override { return this; }
@@ -541,7 +541,7 @@ private:
 // FileStream
 //------------------------------------------------------------------------
 
-#define fileStreamBufSize 256
+#define fileStreamBufSize 16384
 
 class POPPLER_PRIVATE_EXPORT FileStream : public BaseStream
 {
@@ -549,7 +549,7 @@ public:
     FileStream(GooFile *fileA, Goffset startA, bool limitedA, Goffset lengthA, Object &&dictA);
     ~FileStream() override;
     BaseStream *copy() override;
-    Stream *makeSubStream(Goffset startA, bool limitedA, Goffset lengthA, Object &&dictA) override;
+    std::unique_ptr<Stream> makeSubStream(Goffset startA, bool limitedA, Goffset lengthA, Object &&dictA) override;
     StreamKind getKind() const override { return strFile; }
     [[nodiscard]] bool reset() override;
     void close() override;
@@ -615,10 +615,10 @@ private:
 class POPPLER_PRIVATE_EXPORT CachedFileStream : public BaseStream
 {
 public:
-    CachedFileStream(CachedFile *ccA, Goffset startA, bool limitedA, Goffset lengthA, Object &&dictA);
+    CachedFileStream(std::shared_ptr<CachedFile> ccA, Goffset startA, bool limitedA, Goffset lengthA, Object &&dictA);
     ~CachedFileStream() override;
     BaseStream *copy() override;
-    Stream *makeSubStream(Goffset startA, bool limitedA, Goffset lengthA, Object &&dictA) override;
+    std::unique_ptr<Stream> makeSubStream(Goffset startA, bool limitedA, Goffset lengthA, Object &&dictA) override;
     StreamKind getKind() const override { return strCachedFile; }
     [[nodiscard]] bool reset() override;
     void close() override;
@@ -635,7 +635,7 @@ public:
 private:
     bool fillBuf();
 
-    CachedFile *cc;
+    std::shared_ptr<CachedFile> cc;
     Goffset start;
     bool limited;
     char buf[cachedStreamBufSize];
@@ -665,7 +665,7 @@ public:
 
     BaseStream *copy() override { return new BaseMemStream(buf, start, length, dict.copy()); }
 
-    Stream *makeSubStream(Goffset startA, bool limited, Goffset lengthA, Object &&dictA) override
+    std::unique_ptr<Stream> makeSubStream(Goffset startA, bool limited, Goffset lengthA, Object &&dictA) override
     {
         Goffset newLength;
 
@@ -674,7 +674,7 @@ public:
         } else {
             newLength = lengthA;
         }
-        return new BaseMemStream(buf, startA, newLength, std::move(dictA));
+        return std::make_unique<BaseMemStream>(buf, startA, newLength, std::move(dictA));
     }
 
     StreamKind getKind() const override { return strWeird; }
@@ -761,14 +761,15 @@ public:
     ~MemStream() override;
 };
 
-class AutoFreeMemStream : public BaseMemStream<char>
+class AutoFreeMemStream final : public BaseMemStream<const char>
 {
     bool filterRemovalForbidden = false;
+    std::vector<char> m_data;
 
 public:
     // AutoFreeMemStream takes ownership over the buffer.
     // The buffer should be created using gmalloc().
-    AutoFreeMemStream(char *bufA, Goffset startA, Goffset lengthA, Object &&dictA) : BaseMemStream(bufA, startA, lengthA, std::move(dictA)) { }
+    AutoFreeMemStream(std::vector<char> &&data, Object &&dictA) : BaseMemStream(data.data(), 0, data.size(), std::move(dictA)), m_data(std::move(data)) { }
     ~AutoFreeMemStream() override;
 
     // A hack to deal with the strange behaviour of PDFDoc::writeObject().
@@ -792,7 +793,7 @@ public:
     EmbedStream(Stream *strA, Object &&dictA, bool limitedA, Goffset lengthA, bool reusableA = false);
     ~EmbedStream() override;
     BaseStream *copy() override;
-    Stream *makeSubStream(Goffset start, bool limitedA, Goffset lengthA, Object &&dictA) override;
+    std::unique_ptr<Stream> makeSubStream(Goffset start, bool limitedA, Goffset lengthA, Object &&dictA) override;
     StreamKind getKind() const override { return str->getKind(); }
     [[nodiscard]] bool reset() override;
     int getChar() override;
