@@ -16,7 +16,7 @@
 // Copyright (C) 2008 Koji Otani <sho@bbr.jp>
 // Copyright (C) 2012, 2017, 2021, 2023, 2024 Adrian Johnson <ajohnson@redneon.com>
 // Copyright (C) 2012 Hib Eris <hib@hiberis.nl>
-// Copyright (C) 2016, 2018-2022, 2024 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2016, 2018-2022, 2024, 2025 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2016 Jason Crain <jason@aquaticape.us>
 // Copyright (C) 2018 Klarälvdalens Datakonsult AB, a KDAB Group company, <info@kdab.com>. Work sponsored by the LiMux project of the city of Munich
 // Copyright (C) 2018, 2020 Nelson Benítez León <nbenitezl@gmail.com>
@@ -24,6 +24,7 @@
 // Copyright (C) 2023-2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
 // Copyright (C) 2023 Even Rouault <even.rouault@spatialys.com>
 // Copyright (C) 2023, 2024 Oliver Sander <oliver.sander@tu-dresden.de>
+// Copyright (C) 2025 Jonathan Hähne <jonathan.haehne@hotmail.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -40,7 +41,7 @@
 
 #include <config.h>
 
-std::vector<Unicode> UTF16toUCS4(std::span<Unicode> utf16)
+std::vector<Unicode> UTF16toUCS4(std::span<const Unicode> utf16)
 {
     // count characters
     int len = 0;
@@ -406,7 +407,7 @@ std::string utf16ToUtf8(const uint16_t *utf16, int maxUtf16)
     return utf8;
 }
 
-void unicodeToAscii7(std::span<Unicode> in, Unicode **ucs4_out, int *out_len, const int *in_idx, int **indices)
+void unicodeToAscii7(std::span<const Unicode> in, Unicode **ucs4_out, int *out_len, const int *in_idx, int **indices)
 {
     const UnicodeMap *uMap = globalParams->getUnicodeMap("ASCII7");
     int *idx = nullptr;
@@ -461,27 +462,25 @@ void unicodeToAscii7(std::span<Unicode> in, Unicode **ucs4_out, int *out_len, co
 // Convert a PDF Text String to UTF-8
 //   textStr    - PDF text string
 //   returns UTF-8 string.
-std::string TextStringToUtf8(const std::string &textStr)
+std::string TextStringToUtf8(std::string_view textStr)
 {
     int i, len;
-    const char *s;
     std::string utf8;
 
     len = textStr.size();
-    s = textStr.c_str();
     if (hasUnicodeByteOrderMark(textStr)) {
         std::vector<uint16_t> utf16;
         len = len / 2 - 1;
         utf16.resize(len + 1);
         for (i = 0; i < len; i++) {
-            utf16[i] = (s[2 + i * 2] & 0xff) << 8 | (s[3 + i * 2] & 0xff);
+            utf16[i] = (textStr[2 + i * 2] & 0xff) << 8 | (textStr[3 + i * 2] & 0xff);
         }
         utf16[i] = 0;
         utf8 = utf16ToUtf8(utf16.data(), utf16.size());
     } else {
         utf8.resize(len + 1);
         for (i = 0; i < len; i++) {
-            utf8[i] = pdfDocEncoding[s[i] & 0xff];
+            utf8[i] = pdfDocEncoding[textStr[i] & 0xff];
         }
         utf8[i] = 0;
     }
