@@ -15,7 +15,7 @@
 //
 // Copyright (C) 2005 Dan Sheridan <dan.sheridan@postman.org.uk>
 // Copyright (C) 2005 Brad Hards <bradh@frogmouth.net>
-// Copyright (C) 2006, 2008, 2010, 2012-2014, 2016-2024 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2006, 2008, 2010, 2012-2014, 2016-2025 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2007-2008 Julien Rebetez <julienr@svn.gnome.org>
 // Copyright (C) 2007 Carlos Garcia Campos <carlosgc@gnome.org>
 // Copyright (C) 2009, 2010 Ilya Gorenbein <igorenbein@finjan.com>
@@ -115,7 +115,6 @@ private:
 
 ObjectStream::ObjectStream(XRef *xref, int objStrNumA, int recursion)
 {
-    Stream *str;
     Parser *parser;
     Goffset *offsets;
     Object objStr, obj1;
@@ -170,8 +169,8 @@ ObjectStream::ObjectStream(XRef *xref, int objStrNumA, int recursion)
     offsets = (Goffset *)gmallocn(nObjects, sizeof(Goffset));
 
     // parse the header: object numbers and offsets
-    str = new EmbedStream(objStr.getStream(), Object(objNull), true, first);
-    parser = new Parser(xref, str, false);
+    auto embedStr = std::make_unique<EmbedStream>(objStr.getStream(), Object::null(), true, first);
+    parser = new Parser(xref, std::move(embedStr), false);
     for (i = 0; i < nObjects; ++i) {
         obj1 = parser->getObj();
         Object obj2 = parser->getObj();
@@ -192,8 +191,11 @@ ObjectStream::ObjectStream(XRef *xref, int objStrNumA, int recursion)
             return;
         }
     }
-    while (str->getChar() != EOF) {
-        ;
+    {
+        auto str = parser->getStream();
+        while (str && str->getChar() != EOF) {
+            ;
+        }
     }
     delete parser;
 
@@ -206,14 +208,16 @@ ObjectStream::ObjectStream(XRef *xref, int objStrNumA, int recursion)
 
     // parse the objects
     for (i = 0; i < nObjects; ++i) {
+        std::unique_ptr<Stream> strPtr;
         if (i == nObjects - 1) {
-            str = new EmbedStream(objStr.getStream(), Object(objNull), false, 0);
+            strPtr = std::make_unique<EmbedStream>(objStr.getStream(), Object::null(), false, 0);
         } else {
-            str = new EmbedStream(objStr.getStream(), Object(objNull), true, offsets[i + 1] - offsets[i]);
+            strPtr = std::make_unique<EmbedStream>(objStr.getStream(), Object::null(), true, offsets[i + 1] - offsets[i]);
         }
-        parser = new Parser(xref, str, false);
+        parser = new Parser(xref, std::move(strPtr), false);
         objs[i] = parser->getObj();
-        while (str->getChar() != EOF) {
+        auto str = parser->getStream();
+        while (str && str->getChar() != EOF) {
             ;
         }
         delete parser;
@@ -232,7 +236,7 @@ ObjectStream::~ObjectStream()
 Object ObjectStream::getObject(int objIdx, int objNum)
 {
     if (objIdx < 0 || objIdx >= nObjects || objNum != objNums[objIdx]) {
-        return Object(objNull);
+        return Object::null();
     }
     return objs[objIdx].copy();
 }
@@ -401,7 +405,8 @@ XRef *XRef::copy() const
         xref->entries[i].offset = entries[i].offset;
         xref->entries[i].type = entries[i].type;
         // set the object to null, it will be fetched from the stream when needed
-        new (&xref->entries[i].obj) Object(objNull);
+        new (&xref->entries[i].obj) Object();
+        xref->entries[i].obj = Object::null();
         xref->entries[i].flags = entries[i].flags;
         xref->entries[i].gen = entries[i].gen;
 
@@ -468,7 +473,8 @@ int XRef::resize(int newSize)
         for (int i = size; i < newSize; ++i) {
             entries[i].offset = -1;
             entries[i].type = xrefEntryNone;
-            new (&entries[i].obj) Object(objNull);
+            new (&entries[i].obj) Object();
+            entries[i].obj = Object::null();
             entries[i].flags = 0;
             entries[i].gen = 0;
         }
@@ -513,7 +519,7 @@ bool XRef::readXRef(Goffset *pos, std::vector<Goffset> *followedXRefStm, std::ve
     }
 
     // start up a parser, parse one token
-    parser = new Parser(nullptr, str->makeSubStream(parsePos, false, 0, Object(objNull)), true);
+    parser = new Parser(nullptr, str->makeSubStream(parsePos, false, 0, Object::null()), true);
     obj = parser->getObj(true);
 
     // parse an old-style xref table
@@ -941,7 +947,7 @@ bool XRef::constructXRef(bool *wasReconstructed, bool needCatalogDict)
 
             // got trailer dictionary
             if (!strncmp(p, "trailer", 7)) {
-                parser = new Parser(nullptr, str->makeSubStream(pos + 7, false, 0, Object(objNull)), false);
+                parser = new Parser(nullptr, str->makeSubStream(pos + 7, false, 0, Object::null()), false);
                 Object newTrailerDict = parser->getObj();
                 if (newTrailerDict.isDict()) {
                     const Object &obj = newTrailerDict.dictLookupNF("Root");
@@ -1200,11 +1206,11 @@ Object XRef::fetch(int num, int gen, int recursion, Goffset *endPos)
     const Ref ref = { num, gen };
 
     if (!refsBeingFetched.insert(ref)) {
-        return Object(objNull);
+        return Object::null();
     }
 
     // Will remove ref from refsBeingFetched once it's destroyed, i.e. the function returns
-    RefRecursionCheckerRemover remover(refsBeingFetched, ref);
+    auto remover = std::make_unique<RefRecursionCheckerRemover>(refsBeingFetched, ref);
 
     // check for bogus ref - this can happen in corrupted PDF files
     if (num < 0 || num >= size) {
@@ -1222,7 +1228,7 @@ Object XRef::fetch(int num, int gen, int recursion, Goffset *endPos)
         if (e->gen != gen || e->offset < 0) {
             goto err;
         }
-        Parser parser { this, str->makeSubStream(start + e->offset, false, 0, Object(objNull)), true };
+        Parser parser { this, str->makeSubStream(start + e->offset, false, 0, Object::null()), true };
         obj1 = parser.getObj(recursion);
         obj2 = parser.getObj(recursion);
         obj3 = parser.getObj(recursion);
@@ -1301,18 +1307,19 @@ err:
             error(errInternal, -1, "xref num {0:d} not found but needed, document has changes, reconstruct aborted", num);
             // pretend we constructed the xref, otherwise we will do this check again and again
             xrefReconstructed = true;
-            return Object(objNull);
+            return Object::null();
         }
 
         error(errInternal, -1, "xref num {0:d} not found but needed, try to reconstruct", num);
         rootNum = -1;
         constructXRef(&xrefReconstructed);
+        remover.reset(); // Manually delete the remover since we're calling ourselves so recursion for this one is valid
         return fetch(num, gen, ++recursion, endPos);
     }
     if (endPos) {
         *endPos = -1;
     }
-    return Object(objNull);
+    return Object::null();
 }
 
 void XRef::lock()
@@ -1433,7 +1440,8 @@ bool XRef::add(int num, int gen, Goffset offs, bool used)
         for (int i = size; i < num + 1; ++i) {
             entries[i].offset = -1;
             entries[i].type = xrefEntryFree;
-            new (&entries[i].obj) Object(objNull);
+            new (&entries[i].obj) Object();
+            entries[i].obj = Object::null();
             entries[i].flags = 0;
             entries[i].gen = 0;
         }
@@ -1521,10 +1529,10 @@ void XRef::removeIndirectObject(Ref r)
     setModified();
 }
 
-Ref XRef::addStreamObject(Dict *dict, char *buffer, const Goffset bufferSize, StreamCompression compression)
+Ref XRef::addStreamObject(Dict *dict, std::vector<char> buffer, StreamCompression compression)
 {
-    dict->add("Length", Object((int)bufferSize));
-    AutoFreeMemStream *stream = new AutoFreeMemStream(buffer, 0, bufferSize, Object(dict));
+    dict->add("Length", Object((int)buffer.size()));
+    auto stream = std::make_unique<AutoFreeMemStream>(std::move(buffer), Object(dict));
     stream->setFilterRemovalForbidden(true);
     switch (compression) {
     case StreamCompression::None:;
@@ -1533,12 +1541,7 @@ Ref XRef::addStreamObject(Dict *dict, char *buffer, const Goffset bufferSize, St
         stream->getDict()->add("Filter", Object(objName, "FlateDecode"));
         break;
     }
-    return addIndirectObject(Object((Stream *)stream));
-}
-
-Ref XRef::addStreamObject(Dict *dict, uint8_t *buffer, const Goffset bufferSize, StreamCompression compression)
-{
-    return addStreamObject(dict, (char *)buffer, bufferSize, compression);
+    return addIndirectObject(Object(std::move(stream)));
 }
 
 void XRef::writeXRef(XRef::XRefWriter *writer, bool writeAllEntries)
@@ -1685,7 +1688,7 @@ bool XRef::parseEntry(Goffset offset, XRefEntry *entry)
         return false;
     }
 
-    Parser parser(nullptr, str->makeSubStream(offset, false, 20, Object(objNull)), true);
+    Parser parser(nullptr, str->makeSubStream(offset, false, 20, Object::null()), true);
 
     Object obj1, obj2, obj3;
     if (((obj1 = parser.getObj(), obj1.isInt()) || obj1.isInt64()) && (obj2 = parser.getObj(), obj2.isInt()) && (obj3 = parser.getObj(), obj3.isCmd("n") || obj3.isCmd("f"))) {
@@ -1759,7 +1762,7 @@ struct DummyXRefEntry : XRefEntry
         gen = 0;
         type = xrefEntryNone;
         flags = 0;
-        obj = Object(objNull);
+        obj = Object::null();
     }
 };
 
