@@ -14,7 +14,7 @@
 // under GPL version 2 or later
 //
 // Copyright (C) 2005 Brad Hards <bradh@frogmouth.net>
-// Copyright (C) 2005-2009, 2011, 2012, 2014, 2015, 2018, 2019, 2021, 2022, 2024 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2005-2009, 2011, 2012, 2014, 2015, 2018, 2019, 2021, 2022, 2024, 2025 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2008, 2010 Pino Toscano <pino@kde.org>
 // Copyright (C) 2009, 2011 Carlos Garcia Campos <carlosgc@gnome.org>
 // Copyright (C) 2009 Petr Gajdos <pgajdos@novell.com>
@@ -112,7 +112,7 @@ const QPicture &QPainterOutputDevType3Font::getGlyph(int gid) const
         // Glyph has not been rendered before: render it now
 
         // Smallest box that contains all the glyphs from this font
-        const double *fontBBox = m_font->getFontBBox();
+        const std::array<double, 4> &fontBBox = m_font->getFontBBox();
         PDFRectangle box(fontBBox[0], fontBBox[1], fontBBox[2], fontBBox[3]);
 
         Dict *resDict = m_font->getResources();
@@ -515,7 +515,7 @@ void QPainterOutputDev::updateFont(GfxState *state)
 
     } else {
 
-        std::optional<std::vector<unsigned char>> fontBuffer;
+        std::vector<unsigned char> fontBuffer;
 
         std::optional<GfxFontLoc> fontLoc = gfxFont->locateFont(xref, nullptr);
         if (!fontLoc) {
@@ -526,10 +526,11 @@ void QPainterOutputDev::updateFont(GfxState *state)
         // embedded font
         if (fontLoc->locType == gfxFontLocEmbedded) {
             // if there is an embedded font, read it to memory
-            fontBuffer = gfxFont->readEmbFontFile(xref);
-            if (!fontBuffer) {
+            std::optional<std::vector<unsigned char>> fontData = gfxFont->readEmbFontFile(xref);
+            if (!fontData) {
                 return;
             }
+            fontBuffer = std::move(*fontData);
 
             // external font
         } else { // gfxFontLocExternal
@@ -551,7 +552,7 @@ void QPainterOutputDev::updateFont(GfxState *state)
                     return;
                 }
             } else {
-                if (FT_New_Memory_Face(m_ftLibrary, (const FT_Byte *)fontBuffer->data(), fontBuffer->size(), fontLoc->fontNum, &freeTypeFace)) {
+                if (FT_New_Memory_Face(m_ftLibrary, (const FT_Byte *)fontBuffer.data(), fontBuffer.size(), fontLoc->fontNum, &freeTypeFace)) {
                     error(errSyntaxError, -1, "Couldn't create a FreeType face for '{0:s}'", gfxFont->getName() ? gfxFont->getName()->c_str() : "(unnamed)");
                     return;
                 }
@@ -581,7 +582,7 @@ void QPainterOutputDev::updateFont(GfxState *state)
         }
         case fontTrueType:
         case fontTrueTypeOT: {
-            auto ff = (fontLoc->locType != gfxFontLocEmbedded) ? FoFiTrueType::load(fontLoc->path.c_str(), fontLoc->fontNum) : FoFiTrueType::make(std::span(fontBuffer.value()), fontLoc->fontNum);
+            auto ff = (fontLoc->locType != gfxFontLocEmbedded) ? FoFiTrueType::load(fontLoc->path.c_str(), fontLoc->fontNum) : FoFiTrueType::make(std::span(fontBuffer), fontLoc->fontNum);
 
             m_codeToGIDCache[id] = (ff) ? ((Gfx8BitFont *)gfxFont.get())->getCodeToGIDMap(ff.get()) : std::vector<int> {};
 
@@ -593,7 +594,7 @@ void QPainterOutputDev::updateFont(GfxState *state)
 
             // check for a CFF font
             if (!m_useCIDs) {
-                auto ff = (fontLoc->locType != gfxFontLocEmbedded) ? std::unique_ptr<FoFiType1C>(FoFiType1C::load(fontLoc->path.c_str())) : std::unique_ptr<FoFiType1C>(FoFiType1C::make(std::span(fontBuffer.value())));
+                auto ff = (fontLoc->locType != gfxFontLocEmbedded) ? std::unique_ptr<FoFiType1C>(FoFiType1C::load(fontLoc->path.c_str())) : std::unique_ptr<FoFiType1C>(FoFiType1C::make(std::span(fontBuffer)));
 
                 cidToGIDMap = (ff) ? ff->getCIDToGIDMap() : std::vector<int> {};
             }
@@ -612,7 +613,7 @@ void QPainterOutputDev::updateFont(GfxState *state)
             std::vector<int> cidToGIDMap;
 
             if (codeToGID.empty() && !m_useCIDs) {
-                auto ff = (fontLoc->locType != gfxFontLocEmbedded) ? FoFiTrueType::load(fontLoc->path.c_str(), fontLoc->fontNum) : FoFiTrueType::make(std::span(fontBuffer.value()), fontLoc->fontNum);
+                auto ff = (fontLoc->locType != gfxFontLocEmbedded) ? FoFiTrueType::load(fontLoc->path.c_str(), fontLoc->fontNum) : FoFiTrueType::make(std::span(fontBuffer), fontLoc->fontNum);
 
                 if (ff && ff->isOpenTypeCFF()) {
                     cidToGIDMap = ff->getCIDToGIDMap();
@@ -629,7 +630,7 @@ void QPainterOutputDev::updateFont(GfxState *state)
             if (((GfxCIDFont *)gfxFont.get())->getCIDToGIDLen() > 0) {
                 codeToGID = ((GfxCIDFont *)gfxFont.get())->getCIDToGID();
             } else {
-                auto ff = (fontLoc->locType != gfxFontLocEmbedded) ? FoFiTrueType::load(fontLoc->path.c_str(), fontLoc->fontNum) : FoFiTrueType::make(std::span(fontBuffer.value()), fontLoc->fontNum);
+                auto ff = (fontLoc->locType != gfxFontLocEmbedded) ? FoFiTrueType::load(fontLoc->path.c_str(), fontLoc->fontNum) : FoFiTrueType::make(std::span(fontBuffer), fontLoc->fontNum);
                 if (!ff) {
                     return;
                 }
@@ -867,7 +868,7 @@ void QPainterOutputDev::drawChar(GfxState *state, double x, double y, double dx,
         // Make the glyph position the coordinate origin -- that's our center of scaling
         m_painter.top()->translate(QPointF(x - originX, y - originY));
 
-        const double *mat = gfxFont->getFontMatrix();
+        const std::array<double, 6> &mat = gfxFont->getFontMatrix();
         QTransform fontMatrix(mat[0], mat[1], mat[2], mat[3], mat[4], mat[5]);
 
         // Scale with the font size
@@ -1110,7 +1111,7 @@ void QPainterOutputDev::drawSoftMaskedImage(GfxState *state, Object *ref, Stream
     m_painter.top()->drawImage(QRect(0, 0, 1, 1), image);
 }
 
-void QPainterOutputDev::beginTransparencyGroup(GfxState * /*state*/, const double * /*bbox*/, GfxColorSpace * /*blendingColorSpace*/, bool /*isolated*/, bool /*knockout*/, bool /*forSoftMask*/)
+void QPainterOutputDev::beginTransparencyGroup(GfxState * /*state*/, const std::array<double, 4> & /*bbox*/, GfxColorSpace * /*blendingColorSpace*/, bool /*isolated*/, bool /*knockout*/, bool /*forSoftMask*/)
 {
     // The entire transparency group will be painted into a
     // freshly created QPicture object.  Since an existing painter
@@ -1139,7 +1140,7 @@ void QPainterOutputDev::endTransparencyGroup(GfxState * /*state*/)
     m_qpictures.pop();
 }
 
-void QPainterOutputDev::paintTransparencyGroup(GfxState * /*state*/, const double * /*bbox*/)
+void QPainterOutputDev::paintTransparencyGroup(GfxState * /*state*/, const std::array<double, 4> & /*bbox*/)
 {
     // Actually draw the transparency group
     m_painter.top()->drawPicture(0, 0, *m_lastTransparencyGroupPicture);
