@@ -1,7 +1,8 @@
 /* poppler-link-private.h: qt interface to poppler
- * Copyright (C) 2016, 2018, 2020 Albert Astals Cid <aacid@kde.org>
+ * Copyright (C) 2016, 2018, 2020, 2021 Albert Astals Cid <aacid@kde.org>
  * Copyright (C) 2018 Intevation GmbH <intevation@intevation.de>
  * Copyright (C) 2020, 2021 Oliver Sander <oliver.sander@tu-dresden.de>
+ * Copyright (C) 2024 Pratham Gandhi <ppg.1382@gmail.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -23,6 +24,7 @@
 
 #include <vector>
 
+#include "poppler-link.h"
 #include "Link.h"
 
 class LinkOCGState;
@@ -34,7 +36,7 @@ class Link;
 class LinkPrivate
 {
 public:
-    LinkPrivate(const QRectF &area) : linkArea(area) { }
+    explicit LinkPrivate(const QRectF &area) : linkArea(area) { }
 
     virtual ~LinkPrivate();
 
@@ -67,6 +69,27 @@ public:
     bool isShow;
 };
 
+class LinkResetFormPrivate : public LinkPrivate
+{
+public:
+    LinkResetFormPrivate(const QRectF &area, const QStringList &fields, const bool exclude) : LinkPrivate(area), m_fields(fields), m_exclude(exclude) { }
+    ~LinkResetFormPrivate() override;
+
+    QStringList m_fields;
+    bool m_exclude;
+};
+
+class LinkSubmitFormPrivate : public LinkPrivate
+{
+public:
+    LinkSubmitFormPrivate(const QRectF &area, const QVector<int> &fieldIds, const QString &url, LinkSubmitForm::SubmitFormFlags flags) : LinkPrivate(area), m_fieldIds(fieldIds), m_url(url), m_flags(flags) { }
+    ~LinkSubmitFormPrivate() override;
+
+    QVector<int> m_fieldIds;
+    QString m_url;
+    LinkSubmitForm::SubmitFormFlags m_flags;
+};
+
 }
 
 #endif
