@@ -2,7 +2,7 @@
 #include <memory>
 #include <sstream>
 
-#include <QtTest/QtTest>
+#include <QtTest/QTest>
 #include <QTemporaryFile>
 
 #include <poppler-qt6.h>
@@ -19,7 +19,7 @@ public:
 
     void saveAndCheck(const std::unique_ptr<Poppler::Document> &doc, const std::function<void(Poppler::Annotation *a)> &checkFunction);
 
-private slots:
+private Q_SLOTS:
     void checkQColorPrecision();
     void checkFontSizeAndColor();
     void checkHighlightFromAndToQuads();
@@ -33,13 +33,12 @@ private slots:
 void TestAnnotations::checkQColorPrecision()
 {
     bool precisionOk = true;
-    for (int i = std::numeric_limits<uint16_t>::min(); i <= std::numeric_limits<uint16_t>::max(); i++) {
-        double normalized = static_cast<uint16_t>(i) / static_cast<double>(std::numeric_limits<uint16_t>::max());
-        GooString *serialized = GooString::format("{0:.5f}", normalized);
-        double deserialized = gatof(serialized->c_str());
-        delete serialized;
+    for (uint32_t i = std::numeric_limits<uint16_t>::min(); i <= std::numeric_limits<uint16_t>::max(); i++) {
+        double normalized = i / static_cast<double>(std::numeric_limits<uint16_t>::max());
+        const std::string serialized = GooString::format("{0:.5f}", normalized);
+        double deserialized = gatof(serialized.c_str());
         uint16_t denormalized = std::round(deserialized * std::numeric_limits<uint16_t>::max());
-        if (static_cast<uint16_t>(i) != denormalized) {
+        if (i != denormalized) {
             precisionOk = false;
             break;
         }
@@ -58,7 +57,7 @@ void TestAnnotations::checkFontSizeAndColor()
     tempFile.close();
 
     {
-        std::unique_ptr<Poppler::Document> doc { Poppler::Document::load(TESTDATADIR "/unittestcases/UseNone.pdf") };
+        std::unique_ptr<Poppler::Document> doc { Poppler::Document::load(QStringLiteral(TESTDATADIR "/unittestcases/UseNone.pdf")) };
         QVERIFY(doc.get());
 
         std::unique_ptr<Poppler::Page> page { doc->page(0) };
@@ -98,8 +97,9 @@ void TestAnnotations::checkFontSizeAndColor()
             QCOMPARE(textAnnot->textFont().pointSize(), testFont.pointSize());
             QCOMPARE(static_cast<int>(textAnnot->textColor().spec()), static_cast<int>(color.spec()));
             QCOMPARE(textAnnot->textColor(), color);
-            if (annot != annots.cend())
+            if (annot != annots.cend()) {
                 ++annot;
+            }
         }
     }
 }
@@ -115,7 +115,7 @@ static bool operator==(const Poppler::HighlightAnnotation::Quad &a, const Popple
 
 void TestAnnotations::checkHighlightFromAndToQuads()
 {
-    std::unique_ptr<Poppler::Document> doc { Poppler::Document::load(TESTDATADIR "/unittestcases/UseNone.pdf") };
+    std::unique_ptr<Poppler::Document> doc { Poppler::Document::load(QStringLiteral(TESTDATADIR "/unittestcases/UseNone.pdf")) };
 
     std::unique_ptr<Poppler::Page> page { doc->page(0) };
 
@@ -129,7 +129,7 @@ void TestAnnotations::checkHighlightFromAndToQuads()
 
 void TestAnnotations::checkUTF16LEAnnot()
 {
-    std::unique_ptr<Poppler::Document> doc { Poppler::Document::load(TESTDATADIR "/unittestcases/utf16le-annot.pdf") };
+    std::unique_ptr<Poppler::Document> doc { Poppler::Document::load(QStringLiteral(TESTDATADIR "/unittestcases/utf16le-annot.pdf")) };
     QVERIFY(doc.get());
 
     std::unique_ptr<Poppler::Page> page { doc->page(0) };
@@ -162,7 +162,7 @@ void TestAnnotations::saveAndCheck(const std::unique_ptr<Poppler::Document> &doc
 
 void TestAnnotations::checkModificationCreationDate()
 {
-    std::unique_ptr<Poppler::Document> doc { Poppler::Document::load(TESTDATADIR "/unittestcases/utf16le-annot.pdf") };
+    std::unique_ptr<Poppler::Document> doc { Poppler::Document::load(QStringLiteral(TESTDATADIR "/unittestcases/utf16le-annot.pdf")) };
     QVERIFY(doc.get());
 
     std::unique_ptr<Poppler::Page> page { doc->page(0) };
@@ -209,7 +209,7 @@ void TestAnnotations::checkModificationCreationDate()
 
 void TestAnnotations::checkNonMarkupAnnotations()
 {
-    std::unique_ptr<Poppler::Document> doc { Poppler::Document::load(TESTDATADIR "/unittestcases/checkbox_issue_159.pdf") };
+    std::unique_ptr<Poppler::Document> doc { Poppler::Document::load(QStringLiteral(TESTDATADIR "/unittestcases/checkbox_issue_159.pdf")) };
     QVERIFY(doc.get());
 
     std::unique_ptr<Poppler::Page> page { doc->page(0) };
