@@ -1,4 +1,4 @@
-#include <QtTest/QtTest>
+#include <QtTest/QTest>
 
 #include <poppler-qt6.h>
 
@@ -6,14 +6,14 @@ class TestPermissions : public QObject
 {
     Q_OBJECT
 public:
-    TestPermissions(QObject *parent = nullptr) : QObject(parent) { }
-private slots:
+    explicit TestPermissions(QObject *parent = nullptr) : QObject(parent) { }
+private Q_SLOTS:
     void permissions1();
 };
 
 void TestPermissions::permissions1()
 {
-    std::unique_ptr<Poppler::Document> doc = Poppler::Document::load(TESTDATADIR "/unittestcases/orientation.pdf");
+    std::unique_ptr<Poppler::Document> doc = Poppler::Document::load(QStringLiteral(TESTDATADIR "/unittestcases/orientation.pdf"));
     QVERIFY(doc);
 
     // we are allowed to print
