@@ -14,7 +14,7 @@
 // Copyright (C) 2008, 2010, 2014, 2018, 2019, 2021, 2022, 2024 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2010 Paweł Wiejacha <pawel.wiejacha@gmail.com>
 // Copyright (C) 2013, 2014, 2021 Thomas Freitag <Thomas.Freitag@alfa.de>
-// Copyright (C) 2018 Stefan Brüns <stefan.bruens@rwth-aachen.de>
+// Copyright (C) 2018, 2025 Stefan Brüns <stefan.bruens@rwth-aachen.de>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -26,6 +26,7 @@
 #include <cstdlib>
 #include <cstring>
 #include <algorithm>
+#include <limits>
 #include "goo/gmem.h"
 #include "goo/GooLikely.h"
 #include "SplashMath.h"
@@ -39,75 +40,73 @@
 // SplashXPathScanner
 //------------------------------------------------------------------------
 
-SplashXPathScanner::SplashXPathScanner(const SplashXPath &xPath, bool eoA, int clipYMin, int clipYMax)
+SplashXPathScanner::SplashXPathScanner(const SplashXPath &xPath, bool eoA, int clipYMin, int clipYMax) //
+    : eo(eoA), xMin(1), yMin(1), xMax(0), yMax(0)
 {
-    const SplashXPathSeg *seg;
-    SplashCoord xMinFP, yMinFP, xMaxFP, yMaxFP;
-    int i;
-
-    eo = eoA;
-    partialClip = false;
-
     // compute the bbox
-    xMin = yMin = 1;
-    xMax = yMax = 0;
-    if (xPath.length > 0) {
-        seg = &xPath.segs[0];
+    if (xPath.length == 0) {
+        return;
+    }
+    if (clipYMin > clipYMax) {
+        return;
+    }
+
+    SplashCoord xMaxFP = std::numeric_limits<SplashCoord>::lowest();
+    SplashCoord xMinFP = std::numeric_limits<SplashCoord>::max();
+    SplashCoord yMaxFP = std::numeric_limits<SplashCoord>::lowest();
+    SplashCoord yMinFP = std::numeric_limits<SplashCoord>::max();
+
+    SplashCoord clipYMinFP = clipYMin;
+    SplashCoord clipYMaxFP = clipYMax + 1.0;
+
+    for (int i = 0; i < xPath.length; ++i) {
+        const SplashXPathSeg *seg = &xPath.segs[i];
         if (unlikely(std::isnan(seg->x0) || std::isnan(seg->x1) || std::isnan(seg->y0) || std::isnan(seg->y1))) {
             return;
         }
-        if (seg->x0 <= seg->x1) {
+
+        const SplashCoord segYMin = seg->y0;
+        const SplashCoord segYMax = seg->y1;
+        if (segYMin >= clipYMaxFP) {
+            continue;
+        }
+        if (segYMax < clipYMinFP) {
+            continue;
+        }
+
+        if (segYMin < yMinFP) {
+            yMinFP = segYMin;
+        }
+        if (segYMax > yMaxFP) {
+            yMaxFP = segYMax;
+        }
+        if (seg->x0 < xMinFP) {
             xMinFP = seg->x0;
-            xMaxFP = seg->x1;
-        } else {
-            xMinFP = seg->x1;
+        }
+        if (seg->x0 > xMaxFP) {
             xMaxFP = seg->x0;
         }
-        if (seg->flags & splashXPathFlip) {
-            yMinFP = seg->y1;
-            yMaxFP = seg->y0;
-        } else {
-            yMinFP = seg->y0;
-            yMaxFP = seg->y1;
+        if (seg->x1 < xMinFP) {
+            xMinFP = seg->x1;
         }
-        for (i = 1; i < xPath.length; ++i) {
-            seg = &xPath.segs[i];
-            if (unlikely(std::isnan(seg->x0) || std::isnan(seg->x1) || std::isnan(seg->y0) || std::isnan(seg->y1))) {
-                return;
-            }
-            if (seg->x0 < xMinFP) {
-                xMinFP = seg->x0;
-            } else if (seg->x0 > xMaxFP) {
-                xMaxFP = seg->x0;
-            }
-            if (seg->x1 < xMinFP) {
-                xMinFP = seg->x1;
-            } else if (seg->x1 > xMaxFP) {
-                xMaxFP = seg->x1;
-            }
-            if (seg->flags & splashXPathFlip) {
-                if (seg->y0 > yMaxFP) {
-                    yMaxFP = seg->y0;
-                }
-            } else {
-                if (seg->y1 > yMaxFP) {
-                    yMaxFP = seg->y1;
-                }
-            }
-        }
-        xMin = splashFloor(xMinFP);
-        xMax = splashFloor(xMaxFP);
-        yMin = splashFloor(yMinFP);
-        yMax = splashFloor(yMaxFP);
-        if (clipYMin > yMin) {
-            yMin = clipYMin;
-            partialClip = true;
-        }
-        if (clipYMax < yMax) {
-            yMax = clipYMax;
-            partialClip = true;
+        if (seg->x1 > xMaxFP) {
+            xMaxFP = seg->x1;
         }
     }
+    if (yMinFP > yMaxFP) {
+        return;
+    }
+
+    xMin = splashFloor(xMinFP);
+    xMax = splashFloor(xMaxFP);
+    yMin = splashFloor(yMinFP);
+    yMax = splashFloor(yMaxFP);
+    if (clipYMin > yMin) {
+        yMin = clipYMin;
+    }
+    if (clipYMax < yMax) {
+        yMax = clipYMax;
+    }
 
     computeIntersections(xPath);
 }
@@ -122,29 +121,6 @@ void SplashXPathScanner::getBBoxAA(int *xMinA, int *yMinA, int *xMaxA, int *yMax
     *yMaxA = yMax / splashAASize;
 }
 
-void SplashXPathScanner::getSpanBounds(int y, int *spanXMin, int *spanXMax) const
-{
-    if (y < yMin || y > yMax) {
-        *spanXMin = xMax + 1;
-        *spanXMax = xMax;
-        return;
-    }
-    const auto &line = allIntersections[y - yMin];
-    if (!line.empty()) {
-        *spanXMin = line[0].x0;
-        int xx = line[0].x1;
-        for (const SplashIntersect &intersect : line) {
-            if (intersect.x1 > xx) {
-                xx = intersect.x1;
-            }
-        }
-        *spanXMax = xx;
-    } else {
-        *spanXMin = xMax + 1;
-        *spanXMax = xMax;
-    }
-}
-
 bool SplashXPathScanner::test(int x, int y) const
 {
     if (y < yMin || y > yMax) {
@@ -176,11 +152,12 @@ bool SplashXPathScanner::testSpan(int x0, int x1, int y) const
 
     // invariant: the subspan [x0,xx1] is inside the path
     int xx1 = x0 - 1;
+    int eoMask = eo ? 0x1 : ~0;
     while (xx1 < x1) {
         if (i >= line.size()) {
             return false;
         }
-        if (line[i].x0 > xx1 + 1 && !(eo ? (count & 1) : (count != 0))) {
+        if (line[i].x0 > xx1 + 1 && !(count & eoMask)) {
             return false;
         }
         if (line[i].x1 > xx1) {
@@ -200,11 +177,12 @@ bool SplashXPathScanIterator::getNextSpan(int *x0, int *x1)
     if (interIdx >= line.size()) {
         return false;
     }
+    int eoMask = eo ? 0x1 : ~0;
     xx0 = line[interIdx].x0;
     xx1 = line[interIdx].x1;
     interCount += line[interIdx].count;
     ++interIdx;
-    while (interIdx < line.size() && (line[interIdx].x0 <= xx1 || (eo ? (interCount & 1) : (interCount != 0)))) {
+    while (interIdx < line.size() && (line[interIdx].x0 <= xx1 || (interCount & eoMask))) {
         if (line[interIdx].x1 > xx1) {
             xx1 = line[interIdx].x1;
         }
@@ -227,50 +205,43 @@ SplashXPathScanIterator::SplashXPathScanIterator(const SplashXPathScanner &scann
 
 void SplashXPathScanner::computeIntersections(const SplashXPath &xPath)
 {
-    const SplashXPathSeg *seg;
-    SplashCoord segXMin, segXMax, segYMin, segYMax, xx0, xx1;
-    int x, y, y0, y1, i;
-
-    if (yMin > yMax) {
-        return;
-    }
-
     // build the list of all intersections
     allIntersections.resize(yMax - yMin + 1);
 
-    for (i = 0; i < xPath.length; ++i) {
-        seg = &xPath.segs[i];
-        if (seg->flags & splashXPathFlip) {
-            segYMin = seg->y1;
-            segYMax = seg->y0;
-        } else {
-            segYMin = seg->y0;
-            segYMax = seg->y1;
+    const SplashCoord clipYMinFP = yMin;
+    const SplashCoord clipYMaxFP = yMax + 1.0;
+
+    for (int i = 0; i < xPath.length; ++i) {
+        const SplashXPathSeg *seg = &xPath.segs[i];
+        const SplashCoord segYMin = seg->y0;
+        const SplashCoord segYMax = seg->y1;
+        if (segYMin >= clipYMaxFP) {
+            continue;
         }
-        if (seg->flags & splashXPathHoriz) {
-            y = splashFloor(seg->y0);
-            if (y >= yMin && y <= yMax) {
-                if (!addIntersection(segYMin, segYMax, y, splashFloor(seg->x0), splashFloor(seg->x1), 0)) {
-                    break;
-                }
-            }
+        if (segYMax < clipYMinFP) {
+            continue;
+        }
+
+        int y1 = splashFloor(segYMax);
+        int y0 = (seg->flags & splashXPathHoriz) ? y1 : splashFloor(segYMin);
+
+        if (y1 == y0) {
+            addIntersection(segYMin, y1, splashFloor(seg->x0), splashFloor(seg->x1), 0);
         } else if (seg->flags & splashXPathVert) {
-            y0 = splashFloor(segYMin);
             if (y0 < yMin) {
                 y0 = yMin;
             }
-            y1 = splashFloor(segYMax);
             if (y1 > yMax) {
                 y1 = yMax;
             }
-            x = splashFloor(seg->x0);
-            int count = eo || (seg->flags & splashXPathFlip) ? 1 : -1;
-            for (y = y0; y <= y1; ++y) {
-                if (!addIntersection(segYMin, segYMax, y, x, x, count)) {
-                    break;
-                }
+            int x = splashFloor(seg->x0);
+            int count = (seg->flags & splashXPathFlipped) ? 1 : -1;
+            for (int y = y0; y <= y1; ++y) {
+                addIntersection(segYMin, y, x, x, count);
             }
         } else {
+            SplashCoord segXMin, segXMax;
+
             if (seg->x0 < seg->x1) {
                 segXMin = seg->x0;
                 segXMax = seg->x1;
@@ -278,19 +249,18 @@ void SplashXPathScanner::computeIntersections(const SplashXPath &xPath)
                 segXMin = seg->x1;
                 segXMax = seg->x0;
             }
-            y0 = splashFloor(segYMin);
-            if (y0 < yMin) {
-                y0 = yMin;
-            }
-            y1 = splashFloor(segYMax);
-            if (y1 > yMax) {
-                y1 = yMax;
-            }
-            int count = eo || (seg->flags & splashXPathFlip) ? 1 : -1;
             // Calculate the projected intersection of the segment with the
             // X-Axis.
             SplashCoord xbase = seg->x0 - (seg->y0 * seg->dxdy);
-            xx0 = xbase + ((SplashCoord)y0) * seg->dxdy;
+            SplashCoord xx0 = seg->x0;
+            if (y0 < yMin) {
+                y0 = yMin;
+                xx0 = xbase + ((SplashCoord)y0) * seg->dxdy;
+            }
+            if (y1 > yMax) {
+                y1 = yMax;
+            }
+            int count = (seg->flags & splashXPathFlipped) ? 1 : -1;
             // the segment may not actually extend to the top and/or bottom edges
             if (xx0 < segXMin) {
                 xx0 = segXMin;
@@ -299,8 +269,8 @@ void SplashXPathScanner::computeIntersections(const SplashXPath &xPath)
             }
             int x0 = splashFloor(xx0);
 
-            for (y = y0; y <= y1; ++y) {
-                xx1 = xbase + ((SplashCoord)(y + 1) * seg->dxdy);
+            for (int y = y0; y <= y1; ++y) {
+                SplashCoord xx1 = xbase + ((SplashCoord)(y + 1) * seg->dxdy);
 
                 if (xx1 < segXMin) {
                     xx1 = segXMin;
@@ -308,11 +278,8 @@ void SplashXPathScanner::computeIntersections(const SplashXPath &xPath)
                     xx1 = segXMax;
                 }
                 int x1 = splashFloor(xx1);
-                if (!addIntersection(segYMin, segYMax, y, x0, x1, count)) {
-                    break;
-                }
+                addIntersection(segYMin, y, x0, x1, count);
 
-                xx0 = xx1;
                 x0 = x1;
             }
         }
@@ -322,10 +289,9 @@ void SplashXPathScanner::computeIntersections(const SplashXPath &xPath)
     }
 }
 
-inline bool SplashXPathScanner::addIntersection(double segYMin, double segYMax, int y, int x0, int x1, int count)
+inline void SplashXPathScanner::addIntersection(double segYMin, int y, int x0, int x1, int count)
 {
     SplashIntersect intersect;
-    intersect.y = y;
     if (x0 < x1) {
         intersect.x0 = x0;
         intersect.x1 = x1;
@@ -333,36 +299,41 @@ inline bool SplashXPathScanner::addIntersection(double segYMin, double segYMax,
         intersect.x0 = x1;
         intersect.x1 = x0;
     }
-    if (segYMin <= y && (SplashCoord)y < segYMax) {
+    if (segYMin < y) {
         intersect.count = count;
     } else {
         intersect.count = 0;
     }
 
     auto &line = allIntersections[y - yMin];
-#ifndef USE_BOOST_HEADERS
     if (line.empty()) {
+#ifndef USE_BOOST_HEADERS
         line.reserve(4);
-    }
 #endif
-    line.push_back(intersect);
-
-    return true;
+        line.push_back(intersect);
+    } else {
+        auto &last = line.back();
+        // Check if last and new overlap/touch
+        if ((last.x1 + 1) < intersect.x0) {
+            line.push_back(intersect);
+        } else if (last.x0 > (intersect.x1 + 1)) {
+            line.push_back(intersect);
+        } else {
+            last.count += intersect.count;
+            last.x0 = last.x0 < intersect.x0 ? last.x0 : intersect.x0;
+            last.x1 = last.x1 > intersect.x1 ? last.x1 : intersect.x1;
+        }
+    }
 }
 
 void SplashXPathScanner::renderAALine(SplashBitmap *aaBuf, int *x0, int *x1, int y, bool adjustVertLine) const
 {
-    int xx0, xx1, xx, xxMin, xxMax, yy, yyMax, interCount;
-    size_t interIdx;
-    unsigned char mask;
-    SplashColorPtr p;
-
     memset(aaBuf->getDataPtr(), 0, aaBuf->getRowSize() * aaBuf->getHeight());
-    xxMin = aaBuf->getWidth();
-    xxMax = -1;
+    int xxMin = aaBuf->getWidth();
+    int xxMax = -1;
     if (yMin <= yMax) {
-        yy = 0;
-        yyMax = splashAASize - 1;
+        int yy = 0;
+        int yyMax = splashAASize - 1;
         // clamp start and end position
         if (yMin > splashAASize * y) {
             yy = yMin - splashAASize * y;
@@ -371,16 +342,17 @@ void SplashXPathScanner::renderAALine(SplashBitmap *aaBuf, int *x0, int *x1, int
             yyMax = yMax - splashAASize * y;
         }
 
+        int eoMask = eo ? 0x1 : ~0;
         for (; yy <= yyMax; ++yy) {
             const auto &line = allIntersections[splashAASize * y + yy - yMin];
-            interIdx = 0;
-            interCount = 0;
+            size_t interIdx = 0;
+            int interCount = 0;
             while (interIdx < line.size()) {
-                xx0 = line[interIdx].x0;
-                xx1 = line[interIdx].x1;
+                int xx0 = line[interIdx].x0;
+                int xx1 = line[interIdx].x1;
                 interCount += line[interIdx].count;
                 ++interIdx;
-                while (interIdx < line.size() && (line[interIdx].x0 <= xx1 || (eo ? (interCount & 1) : (interCount != 0)))) {
+                while (interIdx < line.size() && (line[interIdx].x0 <= xx1 || (interCount & eoMask))) {
                     if (line[interIdx].x1 > xx1) {
                         xx1 = line[interIdx].x1;
                     }
@@ -396,10 +368,10 @@ void SplashXPathScanner::renderAALine(SplashBitmap *aaBuf, int *x0, int *x1, int
                 }
                 // set [xx0, xx1) to 1
                 if (xx0 < xx1) {
-                    xx = xx0;
-                    p = aaBuf->getDataPtr() + yy * aaBuf->getRowSize() + (xx >> 3);
+                    int xx = xx0;
+                    SplashColorPtr p = aaBuf->getDataPtr() + yy * aaBuf->getRowSize() + (xx >> 3);
                     if (xx & 7) {
-                        mask = adjustVertLine ? 0xff : 0xff >> (xx & 7);
+                        unsigned char mask = adjustVertLine ? 0xff : 0xff >> (xx & 7);
                         if (!adjustVertLine && (xx & ~7) == (xx1 & ~7)) {
                             mask &= (unsigned char)(0xff00 >> (xx1 & 7));
                         }
@@ -431,13 +403,8 @@ void SplashXPathScanner::renderAALine(SplashBitmap *aaBuf, int *x0, int *x1, int
 
 void SplashXPathScanner::clipAALine(SplashBitmap *aaBuf, const int *x0, const int *x1, int y) const
 {
-    int xx0, xx1, xx, yy, yyMin, yyMax, interCount;
-    size_t interIdx;
-    unsigned char mask;
-    SplashColorPtr p;
-
-    yyMin = 0;
-    yyMax = splashAASize - 1;
+    int yyMin = 0;
+    int yyMax = splashAASize - 1;
     // clamp start and end position
     if (yMin > splashAASize * y) {
         yyMin = yMin - splashAASize * y;
@@ -445,22 +412,23 @@ void SplashXPathScanner::clipAALine(SplashBitmap *aaBuf, const int *x0, const in
     if (yyMax + splashAASize * y > yMax) {
         yyMax = yMax - splashAASize * y;
     }
-    for (yy = 0; yy < splashAASize; ++yy) {
-        xx = *x0 * splashAASize;
+    int eoMask = eo ? 0x1 : ~0;
+    for (int yy = 0; yy < splashAASize; ++yy) {
+        int xx = *x0 * splashAASize;
         if (yy >= yyMin && yy <= yyMax) {
             const int intersectionIndex = splashAASize * y + yy - yMin;
             if (unlikely(intersectionIndex < 0 || (unsigned)intersectionIndex >= allIntersections.size())) {
                 break;
             }
             const auto &line = allIntersections[intersectionIndex];
-            interIdx = 0;
-            interCount = 0;
+            size_t interIdx = 0;
+            int interCount = 0;
             while (interIdx < line.size() && xx < (*x1 + 1) * splashAASize) {
-                xx0 = line[interIdx].x0;
-                xx1 = line[interIdx].x1;
+                int xx0 = line[interIdx].x0;
+                int xx1 = line[interIdx].x1;
                 interCount += line[interIdx].count;
                 ++interIdx;
-                while (interIdx < line.size() && (line[interIdx].x0 <= xx1 || (eo ? (interCount & 1) : (interCount != 0)))) {
+                while (interIdx < line.size() && (line[interIdx].x0 <= xx1 || (interCount & eoMask))) {
                     if (line[interIdx].x1 > xx1) {
                         xx1 = line[interIdx].x1;
                     }
@@ -472,9 +440,9 @@ void SplashXPathScanner::clipAALine(SplashBitmap *aaBuf, const int *x0, const in
                 }
                 // set [xx, xx0) to 0
                 if (xx < xx0) {
-                    p = aaBuf->getDataPtr() + yy * aaBuf->getRowSize() + (xx >> 3);
+                    SplashColorPtr p = aaBuf->getDataPtr() + yy * aaBuf->getRowSize() + (xx >> 3);
                     if (xx & 7) {
-                        mask = (unsigned char)(0xff00 >> (xx & 7));
+                        unsigned char mask = (unsigned char)(0xff00 >> (xx & 7));
                         if ((xx & ~7) == (xx0 & ~7)) {
                             mask |= 0xff >> (xx0 & 7);
                         }
@@ -493,15 +461,15 @@ void SplashXPathScanner::clipAALine(SplashBitmap *aaBuf, const int *x0, const in
                 }
             }
         }
-        xx0 = (*x1 + 1) * splashAASize;
+        int xx0 = (*x1 + 1) * splashAASize;
         if (xx0 > aaBuf->getWidth()) {
             xx0 = aaBuf->getWidth();
         }
         // set [xx, xx0) to 0
         if (xx < xx0 && xx >= 0) {
-            p = aaBuf->getDataPtr() + yy * aaBuf->getRowSize() + (xx >> 3);
+            SplashColorPtr p = aaBuf->getDataPtr() + yy * aaBuf->getRowSize() + (xx >> 3);
             if (xx & 7) {
-                mask = (unsigned char)(0xff00 >> (xx & 7));
+                unsigned char mask = (unsigned char)(0xff00 >> (xx & 7));
                 if ((xx & ~7) == (xx0 & ~7)) {
                     mask &= 0xff >> (xx0 & 7);
                 }
