@@ -21,7 +21,9 @@
 static int requested_page = 0;
 static gboolean cairo_output = FALSE;
 static gboolean splash_output = FALSE;
+#ifndef G_OS_WIN32
 static gboolean args_are_fds = FALSE;
+#endif
 static const char **file_arguments = nullptr;
 static const GOptionEntry options[] = { { "cairo", 'c', 0, G_OPTION_ARG_NONE, &cairo_output, "Cairo Output Device", nullptr },
                                         { "splash", 's', 0, G_OPTION_ARG_NONE, &splash_output, "Splash Output Device", nullptr },
@@ -29,7 +31,7 @@ static const GOptionEntry options[] = { { "cairo", 'c', 0, G_OPTION_ARG_NONE, &c
 #ifndef G_OS_WIN32
                                         { "fd", 'f', 0, G_OPTION_ARG_NONE, &args_are_fds, "File descriptors", nullptr },
 #endif
-                                        { G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &file_arguments, nullptr, "PDF-FILES…" },
+                                        { G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_FILENAME_ARRAY, static_cast<void *>(&file_arguments), nullptr, "PDF-FILES…" },
                                         {} };
 
 static GList *view_list = nullptr;
@@ -88,7 +90,7 @@ typedef struct
 // Constants and macros
 //------------------------------------------------------------------------
 
-#define xoutRound(x) ((int)(x + 0.5))
+#define xoutRound(x) ((int)((x) + 0.5))
 
 //------------------------------------------------------------------------
 // GDKSplashOutputDev
@@ -100,7 +102,7 @@ GDKSplashOutputDev::GDKSplashOutputDev(GdkScreen *screen, void (*redrawCbkA)(voi
     redrawCbkData = redrawCbkDataA;
 }
 
-GDKSplashOutputDev::~GDKSplashOutputDev() { }
+GDKSplashOutputDev::~GDKSplashOutputDev() = default;
 
 void GDKSplashOutputDev::clear()
 {
@@ -158,11 +160,13 @@ static gboolean drawing_area_draw(GtkWidget *drawing_area, cairo_t *cr, View *vi
         document.height = view->out->getBitmapHeight();
     }
 
-    if (!gdk_cairo_get_clip_rectangle(cr, &clip))
+    if (!gdk_cairo_get_clip_rectangle(cr, &clip)) {
         return FALSE;
+    }
 
-    if (!gdk_rectangle_intersect(&document, &clip, &draw))
+    if (!gdk_rectangle_intersect(&document, &clip, &draw)) {
         return FALSE;
+    }
 
     if (cairo_output) {
         cairo_set_source_surface(cr, view->surface, 0, 0);
@@ -188,8 +192,9 @@ static void view_set_page(View *view, int page)
         w = (int)ceil(width);
         h = (int)ceil(height);
 
-        if (view->surface)
+        if (view->surface) {
             cairo_surface_destroy(view->surface);
+        }
         view->surface = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, w, h);
 
         cr = cairo_create(view->surface);
@@ -221,8 +226,9 @@ static void redraw_callback(void *data)
 
 static void view_free(View *view)
 {
-    if (G_UNLIKELY(!view))
+    if (G_UNLIKELY(!view)) {
         return;
+    }
 
     g_object_unref(view->doc);
     delete view->out;
@@ -235,8 +241,9 @@ static void destroy_window_callback(GtkWindow *window, View *view)
     view_list = g_list_remove(view_list, view);
     view_free(view);
 
-    if (!view_list)
+    if (!view_list) {
         gtk_main_quit();
+    }
 }
 
 static void page_changed_callback(GtkSpinButton *button, View *view)
@@ -277,11 +284,7 @@ static View *view_new(PopplerDocument *doc)
     view->drawing_area = gtk_drawing_area_new();
     sw = gtk_scrolled_window_new(nullptr, nullptr);
     gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-#if GTK_CHECK_VERSION(3, 7, 8)
     gtk_container_add(GTK_CONTAINER(sw), view->drawing_area);
-#else
-    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(sw), view->drawing_area);
-#endif
     gtk_widget_show(view->drawing_area);
 
     gtk_box_pack_end(GTK_BOX(vbox), sw, TRUE, TRUE, 0);
@@ -384,8 +387,9 @@ int main(int argc, char *argv[])
         }
     }
 
-    if (view_list != nullptr)
+    if (view_list != nullptr) {
         gtk_main();
+    }
 
     return 0;
 }
