@@ -162,7 +162,7 @@ HtmlString::HtmlString(GfxState *state, double fontSize, HtmlFontAccu *_fonts) :
 {
     double x, y;
 
-    state->transform(state->getCurX(), state->getCurY(), &x, &y);
+    state->transform(state->getCurTextX(), state->getCurTextY(), &x, &y);
     if (std::shared_ptr<const GfxFont> font = state->getFont()) {
         double ascent = font->getAscent();
         double descent = font->getDescent();
@@ -299,8 +299,8 @@ void HtmlPage::updateFont(GfxState *state)
     if (font && font->getType() == fontType3) {
         // Grab the font size from the font bounding box if possible - remember to
         // scale from the glyph coordinate system.
-        const double *fontBBox = font->getFontBBox();
-        const double *fontMat = font->getFontMatrix();
+        const std::array<double, 4> &fontBBox = font->getFontBBox();
+        const std::array<double, 6> &fontMat = font->getFontMatrix();
         dimLength = (fontBBox[3] - fontBBox[1]) * fontMat[3];
         if (dimLength > 0) {
             fontSize *= dimLength;
@@ -1213,8 +1213,8 @@ void HtmlOutputDev::startPage(int pageNumA, GfxState *state, XRef *xref)
 void HtmlOutputDev::endPage()
 {
     std::unique_ptr<Links> linksList = docPage->getLinks();
-    for (AnnotLink *link : linksList->getLinks()) {
-        doProcessLink(link);
+    for (const std::shared_ptr<AnnotLink> &link : linksList->getLinks()) {
+        doProcessLink(link.get());
     }
 
     pages->conv();
@@ -1549,9 +1549,9 @@ std::unique_ptr<GooString> HtmlOutputDev::getLinkDest(AnnotLink *link)
                 printf(" link to page %d ", destPage);
             }
             if (printHtml) {
-                const char *p = file->c_str() + file->getLength() - 4;
+                const char *p = file->c_str() + file->size() - 4;
                 if (!strcmp(p, ".pdf") || !strcmp(p, ".PDF")) {
-                    file->del(file->getLength() - 4, 4);
+                    file->erase(file->size() - 4, 4);
                     file->append(".html");
                 }
                 file->append('#');
@@ -1572,9 +1572,9 @@ std::unique_ptr<GooString> HtmlOutputDev::getLinkDest(AnnotLink *link)
         if (printHtml) {
             LinkLaunch *ha = (LinkLaunch *)link->getAction();
             std::unique_ptr<GooString> file = std::make_unique<GooString>(ha->getFileName()->c_str());
-            const char *p = file->c_str() + file->getLength() - 4;
+            const char *p = file->c_str() + file->size() - 4;
             if (!strcmp(p, ".pdf") || !strcmp(p, ".PDF")) {
-                file->del(file->getLength() - 4, 4);
+                file->erase(file->size() - 4, 4);
                 file->append(".html");
             }
             if (printCommands) {
