@@ -15,7 +15,7 @@
 // under GPL version 2 or later
 //
 // Copyright (C) 2006 Dom Lachowicz <cinamod@hotmail.com>
-// Copyright (C) 2007-2010, 2012, 2016-2022, 2024 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2007-2010, 2012, 2016-2022, 2024, 2025 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2010 Hib Eris <hib@hiberis.nl>
 // Copyright (C) 2011 Vittal Aithal <vittal.aithal@cognidox.com>
 // Copyright (C) 2012, 2013, 2016-2018, 2021 Adrian Johnson <ajohnson@redneon.com>
@@ -29,6 +29,7 @@
 // Copyright (C) 2019-2021 Oliver Sander <oliver.sander@tu-dresden.de>
 // Copyright (C) 2019 Thomas Fischer <fischer@unix-ag.uni-kl.de>
 // Copyright (C) 2024, 2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+// Copyright (C) 2025 Jonathan Hähne <jonathan.haehne@hotmail.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -417,7 +418,7 @@ static void printUrlList(PDFDoc *doc)
         Page *page = doc->getPage(pg);
         if (page) {
             std::unique_ptr<Links> links = page->getLinks();
-            for (AnnotLink *annot : links->getLinks()) {
+            for (const std::shared_ptr<AnnotLink> &annot : links->getLinks()) {
                 LinkAction *action = annot->getAction();
                 if (action && action->getKind() == actionURI) {
                     LinkURI *linkUri = dynamic_cast<LinkURI *>(action);
@@ -698,7 +699,7 @@ static void printCustomInfo(PDFDoc *doc, const UnicodeMap *uMap)
                     printInfoDate(info.getDict(), "ModDate", "ModDate:         ", uMap);
                 }
             } else {
-                Object obj = dict->lookup(key.c_str());
+                Object obj = dict->lookup(key);
                 if (obj.isString()) {
                     // print key
                     std::vector<Unicode> u = utf8ToUCS4(key);
@@ -709,8 +710,8 @@ static void printCustomInfo(PDFDoc *doc, const UnicodeMap *uMap)
                     }
 
                     // print value
-                    GooString val_str(obj.getString());
-                    printTextString(&val_str, uMap);
+                    auto val_str = obj.getString();
+                    printTextString(val_str, uMap);
                     fputc('\n', stdout);
                 }
             }
@@ -760,7 +761,7 @@ static void printInfo(PDFDoc *doc, const UnicodeMap *uMap, long long filesize, b
         Dict *dict = info.getDict();
         for (i = 0; i < dict->getLength(); i++) {
             std::string key(dict->getKey(i));
-            if (docInfoStandardKeys.find(key) == docInfoStandardKeys.end()) {
+            if (!docInfoStandardKeys.contains(key)) {
                 hasCustom = true;
                 break;
             }
