@@ -16,7 +16,7 @@
 // under GPL version 2 or later
 //
 // Copyright (C) 2006 Dominic Lachowicz <cinamod@hotmail.com>
-// Copyright (C) 2007-2008, 2010, 2011, 2017-2022, 2024 Albert Astals Cid <aacid@kde.org>
+// Copyright (C) 2007-2008, 2010, 2011, 2017-2022, 2024, 2025 Albert Astals Cid <aacid@kde.org>
 // Copyright (C) 2009 Jan Jockusch <jan@jockusch.de>
 // Copyright (C) 2010, 2013 Hib Eris <hib@hiberis.nl>
 // Copyright (C) 2010 Kenneth Berland <ken@hero.com>
@@ -33,6 +33,8 @@
 // Copyright (C) 2019, 2021 Oliver Sander <oliver.sander@tu-dresden.de>
 // Copyright (C) 2021 William Bader <williambader@hotmail.com>
 // Copyright (C) 2022 kVdNi <kVdNi@waqa.eu>
+// Copyright (C) 2025 g10 Code GmbH, Author: Sune Stolborg Vuorela <sune@vuorela.dk>
+// Copyright (C) 2025 Hagen Möbius <hagen.moebius@googlemail.com>
 //
 // To see a description of the changes please see the Changelog file that
 // came with your tarball or type make ChangeLog if you are building from git
@@ -162,6 +164,15 @@ static std::string myXmlTokenReplace(const char *inString)
     return myString;
 }
 
+static constexpr EndOfLineKind defaultEndOfLine()
+{
+#if defined(_WIN32)
+    return eolDOS;
+#else
+    return eolUnix;
+#endif
+}
+
 int main(int argc, char *argv[])
 {
     std::unique_ptr<PDFDoc> doc;
@@ -171,7 +182,7 @@ int main(int argc, char *argv[])
     const UnicodeMap *uMap;
     Object info;
     bool ok;
-    EndOfLineKind textEOL = TextOutputDev::defaultEndOfLine();
+    EndOfLineKind textEOL = defaultEndOfLine();
 
     Win32Console win32Console(&argc, &argv);
 
@@ -270,9 +281,9 @@ int main(int argc, char *argv[])
         error(errCommandLine, -1, "You have to provide an output filename when reading from stdin.");
         return 99;
     } else {
-        const char *p = fileName.c_str() + fileName.getLength() - 4;
+        const char *p = fileName.c_str() + fileName.size() - 4;
         if (!strcmp(p, ".pdf") || !strcmp(p, ".PDF")) {
-            textFileName = std::make_unique<GooString>(fileName.c_str(), fileName.getLength() - 4);
+            textFileName = std::make_unique<GooString>(fileName.c_str(), fileName.size() - 4);
         } else {
             textFileName = fileName.copy();
         }
@@ -418,7 +429,7 @@ static void printInfoString(FILE *f, Dict *infoDict, const char *key, const char
     bool isUnicode;
     Unicode u;
     char buf[9];
-    int i, n;
+    size_t i, n;
 
     Object obj = infoDict->lookup(key);
     if (obj.isString()) {
@@ -431,7 +442,7 @@ static void printInfoString(FILE *f, Dict *infoDict, const char *key, const char
             isUnicode = false;
             i = 0;
         }
-        while (i < obj.getString()->getLength()) {
+        while (i < obj.getString()->size()) {
             if (isUnicode) {
                 u = ((s1->getChar(i) & 0xff) << 8) | (s1->getChar(i + 1) & 0xff);
                 i += 2;
@@ -622,19 +633,18 @@ void printWordBBox(FILE *f, PDFDoc *doc, TextOutputDev *textOut, int first, int
         double hgt = useCropBox ? doc->getPageCropHeight(page) : doc->getPageMediaHeight(page);
         fprintf(f, "  <page width=\"%f\" height=\"%f\">\n", wid, hgt);
         doc->displayPage(textOut, page, resolution, resolution, 0, !useCropBox, useCropBox, false);
-        std::unique_ptr<TextWordList> wordlist = textOut->makeWordList();
-        const int word_length = wordlist != nullptr ? wordlist->getLength() : 0;
-        TextWord *word;
-        double xMinA, yMinA, xMaxA, yMaxA;
-        if (word_length == 0) {
-            fprintf(stderr, "no word list\n");
-        }
+        const std::unique_ptr<TextWordList> wordlist = textOut->makeWordList();
+        const std::vector<TextWord *> &words = wordlist->getWords();
 
-        for (int i = 0; i < word_length; ++i) {
-            word = wordlist->get(i);
-            word->getBBox(&xMinA, &yMinA, &xMaxA, &yMaxA);
-            const std::string myString = myXmlTokenReplace(word->getText()->c_str());
-            fprintf(f, "    <word xMin=\"%f\" yMin=\"%f\" xMax=\"%f\" yMax=\"%f\">%s</word>\n", xMinA, yMinA, xMaxA, yMaxA, myString.c_str());
+        if (words.empty()) {
+            fprintf(stderr, "no word list\n");
+        } else {
+            for (const TextWord *word : words) {
+                double xMinA, yMinA, xMaxA, yMaxA;
+                word->getBBox(&xMinA, &yMinA, &xMaxA, &yMaxA);
+                const std::string myString = myXmlTokenReplace(word->getText()->c_str());
+                fprintf(f, "    <word xMin=\"%f\" yMin=\"%f\" xMax=\"%f\" yMax=\"%f\">%s</word>\n", xMinA, yMinA, xMaxA, yMaxA, myString.c_str());
+            }
         }
         fprintf(f, "  </page>\n");
     }
